import matplotlib as mpl
from matplotlib import rcParams
import matplotlib.transforms as mtransforms
import matplotlib.pyplot as plt


def _exptext(exp="",
             text="",
             loc=0,
             ax=None,
             fontname=None,
             fontsize=None,
             italic=(False, False),
             pad=0):
    """
    pad in units of axis fraction
    """

    _font_size = rcParams['font.size'] if fontsize is None else fontsize
    fontname = 'TeX Gyre Heros' if fontname is None else fontname

    if ax is None:
        ax = plt.gca()

    loc1_dict = {
        0: {'xy': (0.001, 1 + pad),
            'va': 'bottom',
            },
        1: {'xy': (0.05, 0.95 - pad),
            'va': 'top',
            },
    }

    loc2_dict = {
        0: {'xy': (0.001, 1.005 + pad),
            'va': 'bottom',
            },
        1: {'xy': (0.05, 0.9550 - pad),
            'va': 'bottom',
            },
        2: {'xy': (0.05, 0.9450 - pad),
            'va': 'top',
            },
        3: {'xy': (0.05, 0.95 - pad),
            'va': 'top',
            },
    }

    if loc not in [0, 1, 2, 3]:
        raise ValueError("loc must be in {0, 1, 2}:\n"
                         "0 : Above axes, left aligned\n"
                         "1 : Top left corner\n"
                         "2 : Top left corner, multiline\n"
                         "3 : Split EXP above axes, rest of label in"
                         "top left corner")

    def pixel_to_axis(extent, ax=None):
        # Transform pixel bbox extens to axis fractions
        if ax is None:
            ax = plt.gca()

        extent = extent.transformed(ax.transData.inverted())

        def dist(tup):
            return abs(tup[1] - tup[0])

        dimx, dimy = dist(ax.get_xlim()), dist(ax.get_ylim())
        x, y = ax.get_xlim()[0], ax.get_ylim()[0]
        x0, y0, x1, y1 = extent.extents

        return extent.from_extents(
            abs(x0 - x) / dimx,
            abs(y0 - y) / dimy,
            abs(x1 - x) / dimx,
            abs(y1 - y) / dimy)

    if loc in [0, 3]:
        _exp_loc = 0
    else:
        _exp_loc = 1
    _formater = ax.get_yaxis().get_major_formatter()
    if type(mpl.ticker.ScalarFormatter()) == type(_formater) and _exp_loc == 0:
        ax.figure.canvas.draw()
        _sci_box = pixel_to_axis(ax.get_yaxis().offsetText.get_window_extent())
        _sci_offset = _sci_box.width * 1.1
        loc1_dict[_exp_loc]['xy'] = (_sci_offset, loc1_dict[_exp_loc]['xy'][-1])
        if loc == 0:
            loc2_dict[_exp_loc]['xy'] = (_sci_offset, loc2_dict[_exp_loc]['xy'][-1])

    exp = ax.text(*loc1_dict[_exp_loc]['xy'], s=exp,
                  transform=ax.transAxes,
                  ha='left',
                  va=loc1_dict[_exp_loc]['va'],
                  fontsize=_font_size * 1.3,
                  fontweight='bold',
                  fontstyle='italic' if italic[0] else 'normal',
                  fontname=fontname
                  )

    ax.figure.canvas.draw()
    _dpi = ax.figure.dpi
    _exp_xoffset = exp.get_window_extent().width / _dpi * 1.05
    if loc == 0:
        _t = mtransforms.offset_copy(exp._transform,
                                     x=_exp_xoffset,
                                     units='inches', fig=ax.figure)
    elif loc == 1:
        _t = mtransforms.offset_copy(exp._transform,
                                     x=_exp_xoffset,
                                     y=-exp.get_window_extent().height / _dpi,
                                     units='inches', fig=ax.figure)
    elif loc == 2:
        _t = mtransforms.offset_copy(exp._transform,
                                     y=-exp.get_window_extent().height / _dpi,
                                     units='inches', fig=ax.figure)
    elif loc == 3:
        _t = mtransforms.offset_copy(exp._transform, units='inches', fig=ax.figure)

    ax.text(*loc2_dict[loc]['xy'],
            s=text,
            transform=_t,
            ha='left',
            va=loc2_dict[loc]['va'],
            fontsize=_font_size,
            fontname=fontname,
            fontstyle='italic' if italic[1] else 'normal'
            )

    return ax


# Lumi text
def lumitext(text="",
             ax=None,
             fontname=None,
             fontsize=None):

    _font_size = rcParams['font.size'] if fontsize is None else fontsize
    fontname = 'TeX Gyre Heros' if fontname is None else fontname

    if ax is None:
        ax = plt.gca()

    ax.text(x=1, y=1.005, s=text,
            transform=ax.transAxes,
            ha='right',
            va='bottom',
            fontsize=_font_size * 0.95,
            fontweight='normal',
            fontname=fontname,
            )


# Wrapper
def _explabel(ax=None, loc=0, data=False, paper=False, supplementary=False,
              year=2017, lumi=None, llabel=None, rlabel=None, fontname=None,
              fontsize=None, pad=0, exp="", italic=(False, False)):

    # Right label
    if rlabel is not None:
        _lumi = rlabel
    else:
        if lumi is not None:
            _lumi = r'{lumi}, {year} (13 TeV)'.format(lumi=str(lumi) +
                                                      r' $\mathrm{fb^{-1}}$',
                                                      year=str(year))
        else:
            _lumi = '{} (13 TeV)'.format(str(year))

    lumitext(text=_lumi,
             ax=ax,
             fontname=fontname,
             fontsize=fontsize)

    # Left label
    if llabel is not None:
        _label = llabel
    else:
        _label = ""
        if not data:
            _label = " ".join(["Simulation", _label])
        if not paper:
            _label = " ".join([_label, "Preliminary"])
        if supplementary:
            _label = " ".join([_label, "Supplementary"])

        _label = " ".join(_label.split())

    _exptext(exp=exp,
             text=_label,
             loc=loc,
             ax=ax,
             fontname=fontname,
             fontsize=fontsize,
             italic=italic,
             pad=pad)

    return ax



import os
# Import counter
import requests as req

# Get helper functions
from . import cms
from . import atlas
from . import lhcb
from . import plot
from . import label

from . import tools

# Get styles directly, also available within experiment helpers.
from . import styles as style

from .plot import (
    histplot,
    hist2dplot,

    mpl_magic,
    r_align,
    yscale_legend,
    ylow,

    rescale_to_axessize,
    box_aspect,
    make_square_add_cbar,
    append_axes,
    sort_legend
)

# Make __version__ available
try:
    _base_dir = os.path.dirname(os.path.abspath(__file__))
except NameError:
    _base_dir = None

with open(os.path.join(_base_dir, '.VERSION')) as version_file:
    __version__ = version_file.read().strip()

# Make package fonts available to matplotlib
import os
import matplotlib.font_manager as fm

path = os.path.abspath(__file__)
font_path = "/" + "/".join(path.split("/")[:-1]) + "/fonts/"
font_files = fm.findSystemFonts(fontpaths=font_path)
font_list = fm.createFontList(font_files)
fm.fontManager.ttflist.extend(font_list)

# Log submodules
__all__ = [cms, atlas, lhcb, plot, style, tools, label,
           # Log plot functions
           'histplot',
           'hist2dplot',

           'mpl_magic',
           'r_align',
           'yscale_legend',
           'ylow',

           'rescale_to_axessize',
           'box_aspect',
           'make_square_add_cbar',
           'append_axes',
           'sort_legend'
           ]

# Ping import counter for stats
# Check if CI
istravis = os.environ.get('TRAVIS') == 'true'
isactions = os.environ.get('GITHUB_ACTIONS') == 'true'
istests = os.environ.get('RUNNING_PYTEST') == 'true'
if not (istravis | isactions | istests):
    try:
        # This exists solely to justify my work on the package
        # to my boss. If you have concerns, feel free to open
        # an issue
        __ping__ = req.get("https://countimports.pythonanywhere.com/"
                           "count/tag.svg?url=count_mplhep_imports",
                           timeout=1)
    except Exception:
        pass



from cycler import cycler

colors = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd',
          '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']
markers = ['o', 's', 'D', '^', 'v', '<', '>', 'P', 'X', '*']

ROOT = {
    # Plot properties
    "axes.labelsize": 32,
    "axes.linewidth": 2,
    "axes.facecolor": "white",
    # Custom colors
    "axes.prop_cycle": cycler('color', colors) + cycler('marker', markers),
    "axes.formatter.min_exponent": 3,
    "axes.unicode_minus": False,
    # Figure properties
    "figure.figsize": (12, 9),
    "figure.dpi": 100,
    # Outer frame color
    "figure.facecolor": "white",
    "figure.autolayout": True,

    # Set default font to Times New Roman
    "font.family": "serif",
    "font.serif": ["Times New Roman"],
    "font.size": 14,
    "font.weight": 400,

    # Draw the legend on a solid background
    "legend.frameon": True,
    "legend.fancybox": False,
    # Inherit the background color from the plot
    "legend.facecolor": "inherit",
    "legend.numpoints": 1,
    "legend.labelspacing": 0.2,
    "legend.fontsize": 28,
    "legend.title_fontsize": 28,
    # Automatically choose the best location
    "legend.loc": "best",
    # Space between the handles and their labels
    "legend.handletextpad": 0.75,
    # Space between the borders of the plot and the legend
    "legend.borderaxespad": 1.0,
    "legend.edgecolor": "white",

    # Lines settings
    "lines.linewidth": 4,
    "lines.markeredgewidth": 0,
    "lines.markersize": 8,

    # Saved figure settings
    "savefig.bbox": "tight",
    "savefig.pad_inches": 0.1,
    "savefig.format": "pdf",

    # Ticks settings
    "xtick.major.size": 14,
    "xtick.minor.size": 7,
    "xtick.major.width": 2,
    "xtick.minor.width": 2,
    "xtick.major.pad": 10,
    "xtick.minor.pad": 10,
    "xtick.direction": "in",
    "xtick.labelsize": 30,
    "ytick.major.size": 14,
    "ytick.minor.size": 7,
    "ytick.major.width": 2,
    "ytick.minor.width": 2,
    "ytick.major.pad": 10,
    "ytick.minor.pad": 10,
    "ytick.direction": "in",
    "ytick.labelsize": 30,

    # Legend frame border size
    # WARNING: this affects every patch object
    # (i.e. histograms and so on)
    "patch.linewidth": 2,
}

ROOTTex = {
    # Use LaTeX rendering by default
    # (overrides default font)
    "text.usetex": True,
    # Use the LaTeX version of Times New Roman
    "text.latex.preamble": r"\usepackage{mathptmx}",
    "pgf.rcfonts": False
}



# Log styles
from . import styles_lhcb as style
from . import label as label_base
from . label import lumitext
__all__ = [style, lumitext]


# Experiment wrappers:
def lhcbtext(text="", **kwargs):
    return label_base._exptext("LHCb", text=text, italic=(False, False),
                               fontsize=28, fontname="Times New Roman",
                               loc=1, **kwargs)


def lhcblabel(**kwargs):
    return label_base._explabel(exp="LHCb", italic=(False, False),
                                fontsize=28, fontname="Times New Roman",
                                loc=1, **kwargs)


def text(*args, **kwargs):
    return lhcbtext(*args, **kwargs)


def label(**kwargs):
    return lhcblabel(**kwargs)



# Log styles
from . import styles_cms as style
from . import label as label_base
from . label import lumitext
__all__ = [style, lumitext]


# Experiment wrappers:
def cmstext(text, **kwargs):
    return label_base._exptext("CMS", text=text, italic=(False, True), **kwargs)


def cmslabel(**kwargs):
    return label_base._explabel(exp="CMS", italic=(False, True), **kwargs)


def text(*args, **kwargs):
    return cmstext(*args, **kwargs)


def label(**kwargs):
    return cmslabel(**kwargs)



import os
import matplotlib as mpl


# Backup options
def hardcopy_styles():
    path = os.path.abspath(__file__)
    pkg_dir = "/" + "/".join(path.split("/")[:-1])
    os.system('cp -r {}/stylelib/ '.format(pkg_dir) +
              os.path.join(mpl.get_configdir() + '/'))


def hardcopy_fonts():
    path = os.path.abspath(__file__)
    pkg_dir = "/" + "/".join(path.split("/")[:-1])
    os.system('cp -r {}/fonts/firasans/* '.format(pkg_dir) +
              os.path.join(mpl.rcParams['datapath'] + '/fonts/ttf/'))
    os.system('cp -r {}/fonts/firamath/* '.format(pkg_dir) +
              os.path.join(mpl.rcParams['datapath'] + '/fonts/ttf/'))
    os.system('cp -r {}/fonts/texgyreheros/* '.format(pkg_dir) +
              os.path.join(mpl.rcParams['datapath'] + '/fonts/ttf/'))
    os.system('rm ' + os.path.join(mpl.get_cachedir() + '/font*'))



import os
from setuptools import setup
from setuptools.command.install import install
import sys

INSTALL_REQUIRES = [
    'matplotlib<3' if sys.version_info.major < 3 else 'matplotlib~=3.1',
    'numpy>=1.16.0',
    'scipy>=1.1.0',
    'requests~=2.21',
    'packaging',
]

extras_require = {
    'test': ['pytest', 'pytest-mpl', 'papermill~=1.0', 'nteract-scrapbook~=0.3'],
    'develop': ['flake8', 'jupyter', 'bumpversion', 'twine'],
}
extras_require['complete'] = sorted(set(sum(extras_require.values(), [])))


class PostInstallCommand(install):
    # Currently disabled, done on the fly
    """Post-installation for installation mode."""
    """
    def run(self):

        import os
        import matplotlib as mpl
        os.system('cp -r mplhep/stylelib/ ' +
                  os.path.join(mpl.get_configdir() + '/'))
        os.system('cp -r mplhep/fonts/firasans/* ' +
                  os.path.join(mpl.rcParams['datapath'] + '/fonts/ttf/'))
        os.system('cp -r mplhep/fonts/firamath/* ' +
                  os.path.join(mpl.rcParams['datapath'] + '/fonts/ttf/'))
        os.system('cp -r mplhep/fonts/texgyreheros/* ' +
                  os.path.join(mpl.rcParams['datapath'] + '/fonts/ttf/'))
        os.system('rm ' + os.path.join(mpl.get_cachedir() + '/font*'))
        install.run(self)
    """


try:
    base_dir = os.path.dirname(os.path.abspath(__file__))
except NameError:
    base_dir = None

with open(os.path.join(base_dir, 'mplhep', '.VERSION')) as version_file:
    __version__ = version_file.read().strip()

setup(
    name='mplhep',
    version=__version__,
    author='andrzejnovak',
    author_email='novak5andrzej@gmail.com',
    license='MIT License',
    description='Matplotlib styles for HEP',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/andrzejnovak/mplhep/",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    python_requires=">=3.5",
    install_requires=INSTALL_REQUIRES,
    extras_require=extras_require,
    # packages=find_packages(exclude=['tests']),
    # cmdclass= {'install': PostInstallCommand}, # Currently disabled
    packages=['mplhep'],
    include_package_data=True)

# To push on pypi
# python setup.py sdist
# twine upload dist/*
